<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns="urn:WebWorks-Document-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwxta="urn:WebWorks-XmlToolAdapter-Schema"
                              xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwditaconfig="urn:WebWorks-Dita-Configuration-Schema"
                              xmlns:wwtype="urn:WebWorks-Type-Schema"
                              xmlns:wwstyles="urn:WebWorks-Styles-Schema"
                              exclude-result-prefixes="wwdoc wwxta xsl msxsl wwmode wwfilesystem wwditaconfig wwtype wwstyles"
>
 <!-- Paragraph -->
 <!--           -->

 <xsl:template match="*[@wwtype:type = 'Paragraph']" mode="wwmode:paragraph">
  <xsl:param name="ParamNode" select="." />

  <xsl:call-template name="Paragraph">
   <xsl:with-param name="ParamNode" select="$ParamNode" />
  </xsl:call-template>
 </xsl:template>


 <xsl:template match="*" mode="wwmode:paragraph">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template match="text() | comment() | processing-instruction()" mode="wwmode:paragraph">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template name="Paragraph">
  <xsl:param name="ParamNode" />

  <!-- ID -->
  <!--    -->
  <xsl:variable name="VarID">
   <xsl:call-template name="Generate-ID">
    <xsl:with-param name="ParamNode" select="$ParamNode" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Handle @wwtype:markopen -->
  <!--                         -->
  <xsl:if test="string-length($ParamNode/@wwtype:markopen) &gt; 0">
   <wwdoc:Paragraph id="{$VarID}o" stylename="{$ParamNode/@wwtype:markopen}" preserve="true">
   </wwdoc:Paragraph>
  </xsl:if>

  <!-- Stylename -->
  <!--           -->
  <xsl:variable name="VarStylename">
   <xsl:call-template name="Determine-Style-Name">
    <xsl:with-param name="ParamNode" select="$ParamNode" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Determine extents -->
  <!--                   -->
  <xsl:variable name="VarExtentsAsXML">
   <wwtype:Extent position="0" />

   <xsl:for-each select="$ParamNode/node()">
    <xsl:variable name="VarNode" select="." />

    <xsl:if test="($VarNode/@wwtype:type = 'Structure') or ($VarNode/@wwtype:type = 'Paragraph') or ($VarNode/@wwtype:type = 'Table')">
     <wwtype:Extent position="{position()}" />
    </xsl:if>
   </xsl:for-each>

   <wwtype:Extent position="{count($ParamNode/node()) + 1}" />
  </xsl:variable>
  <xsl:variable name="VarExtents" select="msxsl:node-set($VarExtentsAsXML)/*" />

  <!-- Segment by extents -->
  <!--                    -->
  <xsl:for-each select="$VarExtents[position() &lt; last()]">
   <xsl:variable name="VarExtent" select="." />

   <!-- Current Position -->
   <!--                  -->
   <xsl:variable name="VarCurrentPosition" select="$VarExtent/@position" />

   <!-- Following position -->
   <!--                    -->
   <xsl:variable name="VarFollowingPosition" select="$VarExtent/following-sibling::*[1]/@position" />

   <!-- Node -->
   <!--      -->
   <xsl:if test="$VarCurrentPosition &gt; 0">
    <xsl:apply-templates select="$ParamNode/node()[position() = $VarCurrentPosition]" mode="wwmode:content" />
   </xsl:if>

   <!-- Segment ID -->
   <!--            -->
   <xsl:variable name="VarSegmentID">
    <xsl:choose>
     <xsl:when test="$VarCurrentPosition = 0">
      <xsl:value-of select="$VarID" />
     </xsl:when>

     <xsl:otherwise>
      <xsl:value-of select="$VarID" />
      <xsl:text>-</xsl:text>
      <xsl:value-of select="$VarCurrentPosition" />
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>

   <!-- Segment Stylename -->
   <!--                   -->
   <xsl:variable name="VarSegmentStylename">
    <xsl:choose>
     <xsl:when test="($VarCurrentPosition &gt; 0) and (string-length($ParamNode/@wwtype:remainderclass) &gt; 0)">
      <xsl:value-of select="$ParamNode/@wwtype:remainderclass" />
     </xsl:when>

     <xsl:otherwise>
      <xsl:value-of select="$VarStylename" />
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>

   <!-- Paragraph content nodes -->
   <!--                         -->
   <xsl:variable name="VarParagraphContentNodes" select="$ParamNode/node()[(position() &gt; $VarCurrentPosition) and (position() &lt; $VarFollowingPosition)][namespace-uri() != 'urn:WebWorks-Document-Schema']" />
   <xsl:variable name="VarNonEmpty" select="(count($VarParagraphContentNodes) &gt; 0) or ((contains($ParamNode/@class, ' map/topicref ')) and (string-length($ParamNode/@navtitle) &gt; 0))" />
   <xsl:variable name="VarPreserve" select="false()" />
   <xsl:variable name="VarEmitParagraph" select="$VarNonEmpty or $VarPreserve" />
   <xsl:if test="$VarEmitParagraph">
    <!-- Emit Paragraph -->
    <!--                -->
    <wwdoc:Paragraph id="{$VarSegmentID}" stylename="{$VarSegmentStylename}" preserve="{$VarPreserve}">
     <!-- First paragraph content segment handles the fancy stuff -->
     <!--                                                         -->
     <xsl:if test="$VarSegmentID = $VarID">

      <!-- Aliases -->
      <!--         -->
      <xsl:call-template name="Paragraph-Aliases">
       <xsl:with-param name="ParamNode" select="$ParamNode" />
      </xsl:call-template>

      <!-- Number -->
      <!--        -->
      <xsl:copy-of select="$ParamNode/wwdoc:Number[1]" />
     </xsl:if>

     <!-- Handle @navtitle attribute for current paragraph -->
     <!--                                                  -->
     <xsl:if test="($VarCurrentPosition = 0) and (contains($ParamNode/@class, ' map/topicref ')) and (string-length($ParamNode/@navtitle) &gt; 0)">
      <xsl:variable name="VarTextRunID">
       <xsl:call-template name="Generate-ID">
        <xsl:with-param name="ParamNode" select="$ParamNode/@navtitle" />
       </xsl:call-template>
      </xsl:variable>

      <wwdoc:TextRun id="tr{$VarTextRunID}">
       <wwdoc:Text value="{$ParamNode/@navtitle}" />
      </wwdoc:TextRun>
     </xsl:if>

     <!-- Emit %select-atts; -->
     <!--                    -->
     <xsl:call-template name="Paragraph-Select-Attributes-Markers">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>

     <!-- Using DITAVAL to flag or decorate text. Part 1 of 3.                         -->
     <!-- Call templates are here, while the templates are at the bottom of the file.  -->
     <!-- -Lief 14.June.2013                                                           -->
     <xsl:call-template name="Paragraph-Flagging-AltText">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>

     <xsl:call-template name="Paragraph-Flagging-Color">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>

     <xsl:call-template name="Paragraph-Flagging-BackColor">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>
     
     <xsl:call-template name="Paragraph-Flagging-Style-Italics">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>

     <xsl:call-template name="Paragraph-Flagging-Style-Bold">
      <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>

     <!-- End of customization. -->

     <!-- Process paragraph content nodes -->
     <!--                                 -->
     <xsl:for-each select="$VarParagraphContentNodes">
      <xsl:variable name="VarParagraphContentNode" select="." />

      <!-- Determine whitespace handling -->
      <!--                               -->
      <xsl:variable name="VarWhitespaceHandling">
       <xsl:choose>
        <xsl:when test="($VarSegmentID = $VarID) and ($ParamNode/wwdoc:Style/wwdoc:Attribute[@name = 'white-space']/@value = 'pre')">
         <xsl:value-of select="'preserve-all'" />
        </xsl:when>

        <xsl:when test="($VarSegmentID != $VarID) and (count($ParamNode/wwditaconfig:Remainder) = 0) and ($ParamNode/wwdoc:Style/wwdoc:Attribute[@name = 'white-space']/@value = 'pre')">
         <xsl:value-of select="'preserve-all'" />
        </xsl:when>

        <xsl:when test="($VarSegmentID != $VarID) and ($ParamNode/wwditaconfig:Remainder/wwdoc:Style/wwdoc:Attribute[@name = 'white-space']/@value = 'pre')">
         <xsl:value-of select="'preserve-all'" />
        </xsl:when>

        <xsl:otherwise>
         <xsl:choose>
          <xsl:when test="last() = 1">
           <xsl:value-of select="'collapse-all'" />
          </xsl:when>

          <xsl:when test="position() = 1">
           <xsl:value-of select="'preserve-trailing-space'" />
          </xsl:when>

          <xsl:when test="position() = last()">
           <xsl:value-of select="'preserve-leading-space'" />
          </xsl:when>

          <xsl:otherwise>
           <xsl:value-of select="'preserve-space'" />
          </xsl:otherwise>
         </xsl:choose>
        </xsl:otherwise>
       </xsl:choose>
      </xsl:variable>

      <!-- Process node -->
      <!--              -->
      <xsl:apply-templates select="$VarParagraphContentNode" mode="wwmode:paragraph-content">
       <xsl:with-param name="ParamWhitespaceHandling" select="$VarWhitespaceHandling" />
      </xsl:apply-templates>
     </xsl:for-each>

     <!-- First paragraph content segment handles the fancy stuff -->
     <!--                                                         -->
     <xsl:if test="$VarSegmentID = $VarID">
      <!-- Keywords and Index Entries -->
      <!--                            -->
      <xsl:variable name="VarAncestorTopicElements" select="$ParamNode/ancestor::*[contains(@class, ' topic/topic ')]" />
      <xsl:variable name="VarAncestorTopicElement" select="$VarAncestorTopicElements[last()]" />
      <xsl:if test="string-length($VarAncestorTopicElement/@id) &gt; 0">
       <!-- Same paragraph? -->
       <!--                 -->
       <xsl:variable name="VarFirst">
        <xsl:apply-templates select="$ParamNode" mode="wwmode:hierarchy-element-first-of-wwtype">
         <xsl:with-param name="ParamAncestorElement" select="$VarAncestorTopicElement" />
         <xsl:with-param name="ParamType" select="'Paragraph'" />
        </xsl:apply-templates>
       </xsl:variable>
       <xsl:if test="$VarFirst = 'true'">
        <!-- Text run for markers -->
        <!--                      -->
        <wwdoc:TextRun id="tr{$VarSegmentID}">
         <!-- Existing filename markers? -->
         <!--                            -->
         <xsl:variable name="VarMetaDataElements" select="$VarAncestorTopicElement/*[contains(@class, ' topic/prolog ')]/*[contains(@class, ' topic/metadata ')]" />
         <xsl:variable name="VarFilenameMarkersExist">
          <xsl:call-template name="Filename-Markers-Exist">
           <xsl:with-param name="ParamMetaDataElements" select="$VarMetaDataElements" />
          </xsl:call-template>
         </xsl:variable>

         <!-- Filename and Pathname markers -->
         <!--                               -->
         <xsl:if test="$VarFilenameMarkersExist != string(true())">
          <xsl:call-template name="Filename-Marker">
           <xsl:with-param name="ParamNode" select="$ParamNode" />
           <xsl:with-param name="ParamID" select="$VarSegmentID" />
          </xsl:call-template>
         </xsl:if>
         <xsl:call-template name="Pathname-Marker">
          <xsl:with-param name="ParamNode" select="$ParamNode" />
          <xsl:with-param name="ParamID" select="$VarSegmentID" />
         </xsl:call-template>

         <!-- Emit Map Metadata Markers -->
         <!--                           -->
         <xsl:call-template name="Map-Metadata-Markers">
          <xsl:with-param name="ParamElement" select="$VarAncestorTopicElement" />
          <xsl:with-param name="ParamElementClassFilter" select="' topic/topic '" />
         </xsl:call-template>

         <!-- Emit Metadata Markers -->
         <!--                       -->
         <xsl:call-template name="Metadata-Markers">
          <xsl:with-param name="ParamMetaDataElements" select="$VarMetaDataElements" />
         </xsl:call-template>

         <!-- Emit ancestor %select-atts; -->
         <!--                             -->
         <xsl:call-template name="Ancestor-Select-Attributes-Markers">
          <xsl:with-param name="ParamChildElement" select="$ParamNode" />
         </xsl:call-template>
        </wwdoc:TextRun>
       </xsl:if>
      </xsl:if>

      <xsl:if test="contains($ParamNode/parent::*[1]/@class, ' map/map ')">
       <wwdoc:TextRun>
        <xsl:apply-templates select="$ParamNode/following-sibling::*[contains(@class, ' map/topicmeta ')][1]/*[contains(@class, ' topic/othermeta ')]" mode="wwmode:othermeta" />
       </wwdoc:TextRun>
      </xsl:if>

      <!-- Handle @navtitle attributes -->
      <!--                             -->
      <xsl:for-each select="$ParamNode/parent::*/*[contains(@class, ' topic/titlealts ')]/*[contains(@class, ' topic/navtitle ')]">
       <xsl:variable name="VarNavTitleNode" select="." />

       <xsl:variable name="VarMarkerID">
        <xsl:call-template name="Generate-ID">
         <xsl:with-param name="ParamNode" select="$VarNavTitleNode" />
        </xsl:call-template>
       </xsl:variable>

       <wwdoc:TextRun id="tr{$VarMarkerID}">
        <wwdoc:Marker id="{$VarMarkerID}" name="Navigation Title">
         <xsl:apply-templates select="$VarNavTitleNode/node()" mode="wwmode:paragraph-content">
          <xsl:with-param name="ParamWhitespaceHandling" select="'collapse-all'" />
         </xsl:apply-templates>
        </wwdoc:Marker>
       </wwdoc:TextRun>
      </xsl:for-each>
     </xsl:if>
     <!-- Using DITAVAL to flag or decorate text. Part 2 of 3.      -->
     <!-- Inserting a call template for EndFlag text from DITAVAL.  -->
     <!-- -Lief 17.June.2013                                        -->
     <xsl:call-template name="Paragraph-Flagging-AltEndText">
       <xsl:with-param name="ParamParagraph" select="$ParamNode" />
     </xsl:call-template>
     <!-- End of customization. -->
    </wwdoc:Paragraph>
   </xsl:if>
  </xsl:for-each>

  <!-- Handle @wwtype:markclose -->
  <!--                          -->
  <xsl:if test="string-length($ParamNode/@wwtype:markclose) &gt; 0">
   <wwdoc:Paragraph id="{$VarID}c" stylename="{$ParamNode/@wwtype:markclose}" preserve="true">
   </wwdoc:Paragraph>
  </xsl:if>
 </xsl:template>


 <!-- Character Elements -->
 <!--                    -->
 <xsl:template match="*[@wwtype:type = 'Character']" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <xsl:apply-templates select="$ParamNode" mode="wwmode:character">
   <xsl:with-param name="ParamWhitespaceHandling" select="$ParamWhitespaceHandling" />
  </xsl:apply-templates>
 </xsl:template>


 <!-- Note/Footnote Elements -->
 <!--                        -->
 <xsl:template match="*[@wwtype:type = 'Note']" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />

  <!-- ID -->
  <!--    -->
  <xsl:variable name="VarID">
   <xsl:call-template name="Generate-ID">
    <xsl:with-param name="ParamNode" select="$ParamNode" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Note -->
  <!--      -->
  <wwdoc:TextRun id="tr{$VarID}">
   <wwdoc:Note id="fn{$VarID}">
    <xsl:call-template name="Paragraph">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>
   </wwdoc:Note>
  </wwdoc:TextRun>
 </xsl:template>


 <!-- Index Entries -->
 <!--               -->
 <xsl:template match="*[contains(@class, ' topic/indexterm ')]" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <wwdoc:TextRun>
   <xsl:attribute name="id">
    <xsl:text>tr</xsl:text>
    <xsl:call-template name="Generate-ID">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>
   </xsl:attribute>

   <xsl:apply-templates select="$ParamNode" mode="wwmode:index" />
  </wwdoc:TextRun>
 </xsl:template>


 <!-- Graphic Elements -->
 <!--                  -->
 <xsl:template match="*[@wwtype:type = 'Graphic']" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <wwdoc:TextRun>
   <xsl:attribute name="id">
    <xsl:text>tr</xsl:text>
    <xsl:call-template name="Generate-ID">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>
   </xsl:attribute>

   <xsl:apply-templates select="$ParamNode" mode="wwmode:graphic" />
  </wwdoc:TextRun>
 </xsl:template>


 <!-- Marker Elements -->
 <!--                  -->
 <xsl:template match="*[@wwtype:type = 'Marker']" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Link markers are handled elsewhere -->
  <!--                                    -->
  <xsl:variable name="VarLinkNodes" select="$ParamNode/ancestor-or-self::*[contains(@class, ' topic/link ') and (string-length(@href) &gt; 0)]" />
  <xsl:if test="count($VarLinkNodes) = 0">
   <wwdoc:TextRun>
    <xsl:attribute name="id">
     <xsl:text>tr</xsl:text>
     <xsl:call-template name="Generate-ID">
      <xsl:with-param name="ParamNode" select="$ParamNode" />
     </xsl:call-template>
    </xsl:attribute>

    <xsl:apply-templates select="$ParamNode" mode="wwmode:marker" />
   </wwdoc:TextRun>
  </xsl:if>
 </xsl:template>


 <!-- text() -->
 <!--        -->
 <xsl:template match="text()" mode="wwmode:paragraph-content">
  <xsl:param name="ParamNode" select="." />
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Apply whitespace handling -->
  <!--                           -->
  <xsl:variable name="VarWhitespaceHandledString">
   <xsl:call-template name="Apply-Whitespace-Handling">
    <xsl:with-param name="ParamString" select="$ParamNode" />
    <xsl:with-param name="ParamWhitespaceHandling" select="$ParamWhitespaceHandling" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Emit TextRun -->
  <!--              -->
  <xsl:if test="string-length($VarWhitespaceHandledString) &gt; 0">
   <wwdoc:TextRun>
    <xsl:attribute name="id">
     <xsl:call-template name="Generate-ID">
      <xsl:with-param name="ParamNode" select="$ParamNode" />
     </xsl:call-template>
    </xsl:attribute>

    <!-- Link -->
    <!--      -->
    <xsl:call-template name="Link">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>

    <!-- Text Elements -->
    <!--               -->
    <xsl:call-template name="String-To-Text-Elements">
     <xsl:with-param name="ParamString" select="$VarWhitespaceHandledString" />
     <xsl:with-param name="ParamPreserveWhitespace" select="$ParamWhitespaceHandling = 'preserve-all'" />
    </xsl:call-template>

    <!-- Link Markers -->
    <!--              -->
    <xsl:call-template name="Link-Markers">
     <xsl:with-param name="ParamNode" select="$ParamNode" />
    </xsl:call-template>
   </wwdoc:TextRun>
  </xsl:if>
 </xsl:template>


 <!-- Unmatched -->
 <!--           -->
 <xsl:template match="*" mode="wwmode:paragraph-content">
  <xsl:param name="ParamWhitespaceHandling" />

  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <!-- Paragraph Aliases -->
 <!--                   -->

 <xsl:template name="Paragraph-Aliases">
  <xsl:param name="ParamNode" />

  <!-- Determine aliases -->
  <!--                   -->
  <xsl:variable name="VarAliasesAsXML">
   <!-- Alias for paragraph? -->
   <!--                      -->
   <xsl:if test="string-length($ParamNode/@id) &gt; 0">
    <wwdoc:Alias value="{$ParamNode/@id}" />
   </xsl:if>

   <!-- Aliases for parent topics and topicrefs                             -->
   <!-- Duplicates are allowed as the first match will be used for links    -->
   <!-- This addresses issues when generation of a parent topic is disabled -->
   <!--                                                                     -->
   <xsl:variable name="VarAncestorTopicNodes" select="$ParamNode/ancestor::*[(contains(@class, ' map/topicref ')) or (contains(@class, ' topic/topic ')) or (contains(@class, ' topic/section '))]" />
   <xsl:for-each select="$VarAncestorTopicNodes">
    <xsl:variable name="VarAncestorTopicNode" select="." />

    <!-- Emit alias if an id is present -->
    <!--                                -->
    <xsl:if test="string-length($VarAncestorTopicNode/@id) &gt; 0">
     <wwdoc:Alias value="{$VarAncestorTopicNode/@id}" />
    </xsl:if>
   </xsl:for-each>

   <!-- Table, Figure, and Example aliases -->
   <!--                                    -->
   <xsl:if test="contains($ParamNode/@class, ' topic/title ')">
    <xsl:variable name="VarParentNodes" select="$ParamNode/parent::*[(contains(@class, ' topic/table ')) or (contains(@class, ' topic/fig ')) or (contains(@class, ' topic/example '))]" />
    <xsl:for-each select="$VarParentNodes">
     <xsl:variable name="VarParentNode" select="." />

     <!-- Emit alias if an id is present -->
     <!--                                -->
     <xsl:if test="string-length($VarParentNode/@id) &gt; 0">
      <wwdoc:Alias value="{$VarParentNode/@id}" />
     </xsl:if>
    </xsl:for-each>
   </xsl:if>

   <!-- Aliases for contained non-paragraph elements -->
   <!--                                              -->
   <xsl:apply-templates select="$ParamNode/*" mode="wwtype:child-aliases" />
  </xsl:variable>
  <xsl:variable name="VarAliases" select="msxsl:node-set($VarAliasesAsXML)/wwdoc:Alias" />

  <!-- Emit aliases if any exist -->
  <!--                           -->
  <xsl:if test="count($VarAliases[1]) &gt; 0">
   <wwdoc:Aliases>
    <xsl:copy-of select="$VarAliases" />
   </wwdoc:Aliases>
  </xsl:if>
 </xsl:template>


 <xsl:template match="*[(not(contains(@class, ' map/topicref ') or contains(@class, ' topic/topic ') or contains(@class, ' topic/table ') or contains(@class, ' topic/fig ') or contains(@class, ' topic/example '))) and (string-length(@id) &gt; 0) and ((@wwtype:type != 'Paragraph') or (@wwtype:type != 'Note'))]" mode="wwtype:child-aliases">
  <xsl:param name="ParamNode" select="." />

  <wwdoc:Alias value="{$ParamNode/@id}" />

  <xsl:apply-templates mode="wwtype:child-aliases" />
 </xsl:template>


 <xsl:template match="*" mode="wwtype:child-aliases">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <xsl:template match="text() | comment() | processing-instruction()" mode="wwtype:child-aliases">
  <!-- Suppress -->
  <!--          -->
 </xsl:template>


 <!-- DITA %select-atts; Markers -->
 <!--                            -->

 <xsl:template name="Select-Attributes-Markers">
  <xsl:param name="ParamElement" />

  <!-- Determine baseline text run ID -->
  <!--                                -->
  <xsl:variable name="VarTextRunID">
   <xsl:call-template name="Generate-ID">
    <xsl:with-param name="ParamNode" select="$ParamElement" />
   </xsl:call-template>
  </xsl:variable>

  <!-- Emit markers for title attribute if this is a map element -->
  <!--                                                           -->
  <xsl:if test="contains($ParamElement/@class, ' map/map ')">
   <xsl:for-each select="$ParamElement/@title">
    <xsl:variable name="VarTitleAttribute" select="." />

    <wwdoc:Marker id="mma{$VarTextRunID}" name="Map Title">
     <wwdoc:TextRun id="mmatr{$VarTextRunID}">
      <wwdoc:Text value="{$VarTitleAttribute}" />
     </wwdoc:TextRun>
    </wwdoc:Marker>
   </xsl:for-each>
  </xsl:if>

  <!-- Emit markers for all select attributes -->
  <!--                                        -->
  <xsl:for-each select="$ParamElement/@platform | $ParamElement/@product | $ParamElement/@audience | $ParamElement/@otherprops | $ParamElement/@importance | $ParamElement/@rev | $ParamElement/@status">
   <xsl:variable name="VarSelectAttribute" select="." />

   <!-- Split on spaces -->
   <!--                 -->
   <xsl:call-template name="Select-Attributes-Marker">
    <xsl:with-param name="ParamBaseID" select="concat($VarTextRunID, '-', position())" />
    <xsl:with-param name="ParamSplitCount" select="1" />
    <xsl:with-param name="ParamAttributeName" select="local-name($VarSelectAttribute)" />
    <xsl:with-param name="ParamAttributeValue" select="normalize-space($VarSelectAttribute)" />
   </xsl:call-template>
  </xsl:for-each>
 </xsl:template>


 <xsl:template name="Select-Attributes-Marker">
  <xsl:param name="ParamBaseID" />
  <xsl:param name="ParamSplitCount" />
  <xsl:param name="ParamAttributeName" />
  <xsl:param name="ParamAttributeValue" />

  <!-- Attribute non-empty? -->
  <!--                      -->
  <xsl:if test="string-length($ParamAttributeValue) &gt; 0">
   <!-- Split on whitespace -->
   <!--                     -->
   <xsl:variable name="VarPrefix" select="substring-before($ParamAttributeValue, ' ')" />
   <xsl:variable name="VarSuffix" select="substring-after($ParamAttributeValue, ' ')" />

   <xsl:choose>
    <xsl:when test="(string-length($VarPrefix) = 0) and (string-length($VarSuffix) = 0)">
     <!-- Emit marker -->
     <!--             -->
     <wwdoc:Marker id="msa{$ParamBaseID}-{$ParamSplitCount}" name="Select Attribute - {$ParamAttributeName}" optional="true">
      <wwdoc:TextRun id="msatr{$ParamBaseID}-{$ParamSplitCount}">
       <wwdoc:Text value="{$ParamAttributeValue}" />
      </wwdoc:TextRun>
     </wwdoc:Marker>
    </xsl:when>

    <xsl:when test="string-length($VarPrefix) &gt; 0">
     <!-- Emit marker -->
     <!--             -->
     <wwdoc:Marker id="msa{$ParamBaseID}-{$ParamSplitCount}" name="Select Attribute - {$ParamAttributeName}" optional="true">
      <wwdoc:TextRun id="msatr{$ParamBaseID}-{$ParamSplitCount}">
       <wwdoc:Text value="{$VarPrefix}" />
      </wwdoc:TextRun>
     </wwdoc:Marker>

     <!-- Suffix defined? -->
     <!--                 -->
     <xsl:if test="string-length($VarSuffix)&gt; 0">
      <!-- Recurse -->
      <!--         -->
      <xsl:call-template name="Select-Attributes-Marker">
       <xsl:with-param name="ParamBaseID" select="$ParamBaseID" />
       <xsl:with-param name="ParamSplitCount" select="$ParamSplitCount + 1" />
       <xsl:with-param name="ParamAttributeName" select="$ParamAttributeName" />
       <xsl:with-param name="ParamAttributeValue" select="$VarSuffix" />
      </xsl:call-template>
     </xsl:if>
    </xsl:when>
   </xsl:choose>
  </xsl:if>
 </xsl:template>


 <xsl:template name="Paragraph-Select-Attributes-Markers">
  <xsl:param name="ParamParagraph" />

  <!-- Paragraph has select attributes? -->
  <!--                                  -->
  <xsl:variable name="VarHasSelectAttributes" select="($ParamParagraph/@platform) or ($ParamParagraph/@product) or ($ParamParagraph/@audience) or ($ParamParagraph/@otherprops) or ($ParamParagraph/@importance) or ($ParamParagraph/@rev) or ($ParamParagraph/@status)" />
  <xsl:if test="$VarHasSelectAttributes">
   <xsl:variable name="VarTextRunID">
    <xsl:call-template name="Generate-ID">
     <xsl:with-param name="ParamNode" select="$ParamParagraph" />
    </xsl:call-template>
   </xsl:variable>

   <!-- Emit a text run to hold markers -->
   <!--                                 -->
   <wwdoc:TextRun id="trsa{$VarTextRunID}">
    <!-- Emit select attributes markers -->
    <!--                                -->
    <xsl:call-template name="Select-Attributes-Markers">
     <xsl:with-param name="ParamElement" select="$ParamParagraph"/>
    </xsl:call-template>
   </wwdoc:TextRun>
  </xsl:if>
 </xsl:template>


 <xsl:template name="Ancestor-Select-Attributes-Markers">
  <xsl:param name="ParamChildElement" />

  <!-- Locate element to work with -->
  <!--                             -->
  <xsl:variable name="VarElements" select="$ParamChildElement/ancestor::*[(@wwtype:type = 'Structure') or (@wwtype:type = 'Paragraph')]" />
  <xsl:for-each select="$VarElements[1]">
   <xsl:variable name="VarElement" select="$VarElements[last()]" />

   <!-- Locate closest ancestor element -->
   <!--                                 -->
   <xsl:variable name="VarAncestorElements" select="$VarElement/ancestor::*[(@wwtype:type = 'Structure') or (@wwtype:type = 'Paragraph')]" />
   <xsl:variable name="VarAncestorElementsCount" select="count($VarAncestorElements)" />
   <xsl:choose>
    <!-- No more ancestors -->
    <!--                   -->
    <xsl:when test="$VarAncestorElementsCount = 0">
     <!-- Emit select attributes markers -->
     <!--                                -->
     <xsl:call-template name="Select-Attributes-Markers">
      <xsl:with-param name="ParamElement" select="$VarElement" />
     </xsl:call-template>
    </xsl:when>

    <!-- At least one ancestor -->
    <!--                       -->
    <xsl:otherwise>
     <xsl:for-each select="$VarAncestorElements[1]">
      <xsl:variable name="VarAncestorElement" select="$VarAncestorElements[last()]" />

      <!-- Check to see if this is the first element in a segment -->
      <!--                                                        -->
      <xsl:variable name="VarFirst">
       <xsl:apply-templates select="$VarElement" mode="wwmode:hierarchy-element-first-of-wwtype">
        <xsl:with-param name="ParamAncestorElement" select="$VarAncestorElement" />
        <xsl:with-param name="ParamType" select="$VarElement/@wwtype:type" />
       </xsl:apply-templates>
      </xsl:variable>
      <xsl:if test="$VarFirst = 'true'">
       <!-- Emit ancestor %select-atts; -->
       <!--                             -->
       <xsl:call-template name="Ancestor-Select-Attributes-Markers">
        <xsl:with-param name="ParamChildElement" select="$VarElement" />
       </xsl:call-template>

       <!-- Emit select attributes markers -->
       <!--                                -->
       <xsl:call-template name="Select-Attributes-Markers">
        <xsl:with-param name="ParamElement" select="$VarElement" />
       </xsl:call-template>
      </xsl:if>
     </xsl:for-each>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:for-each>
 </xsl:template>


 <!-- Metadata Markers -->
 <!--                  -->

 <xsl:template name="Metadata-Markers">
  <xsl:param name="ParamMetaDataElements" />

  <!-- Collect up keywords elements -->
  <!--                              -->
  <xsl:variable name="VarKeywordsElements" select="$ParamMetaDataElements/*[contains(@class, ' topic/keywords ')]" />

  <!-- Keywords -->
  <!--          -->
  <xsl:apply-templates select="$VarKeywordsElements" mode="wwmode:keywords" />

  <!-- Index markers -->
  <!--               -->
  <xsl:apply-templates select="$VarKeywordsElements/*" mode="wwmode:index" />

  <!-- Prodinfo markers -->
  <!--                   -->
  <xsl:apply-templates select="$ParamMetaDataElements/*[contains(@class, ' topic/prodinfo ')]" mode="wwmode:prodinfo" />

  <!-- Othermeta markers -->
  <!--                   -->
  <xsl:apply-templates select="$ParamMetaDataElements/*[contains(@class, ' topic/othermeta ')]" mode="wwmode:othermeta" />
 </xsl:template>


 <xsl:template name="Map-Metadata-Markers">
  <xsl:param name="ParamElement" />
  <xsl:param name="ParamElementClassFilter" />

  <!-- Locate closest ancestor element -->
  <!--                                 -->
  <xsl:variable name="VarAncestorElements" select="$ParamElement/ancestor::*[contains(@class, ' map/map ') or contains(@class, ' map/topicref ')]" />
  <xsl:for-each select="$VarAncestorElements[1]">
   <xsl:variable name="VarAncestorElement" select="$VarAncestorElements[last()]" />

   <!-- Process ancestor's ancestor metadata markers if this is the first element in a segment -->
   <!--                                                                                        -->
   <xsl:variable name="VarFirst">
    <xsl:apply-templates select="$ParamElement" mode="wwmode:hierarchy-element-first-of-class">
     <xsl:with-param name="ParamAncestorElement" select="$VarAncestorElement" />
     <xsl:with-param name="ParamClassFilter" select="$ParamElementClassFilter" />
    </xsl:apply-templates>
   </xsl:variable>
   <xsl:if test="$VarFirst = 'true'">
    <!-- Emit Map Metadata Markers -->
    <!--                           -->
    <xsl:call-template name="Map-Metadata-Markers">
     <xsl:with-param name="ParamElement" select="$VarAncestorElement" />
     <xsl:with-param name="ParamElementClassFilter" select="' map/topicref '" />
    </xsl:call-template>

    <!-- Emit Metadata Markers -->
    <!--                       -->
    <xsl:variable name="VarMetaDataElements" select="$VarAncestorElement/*[contains(@class, ' map/topicmeta ')]" />
    <xsl:call-template name="Metadata-Markers">
     <xsl:with-param name="ParamMetaDataElements" select="$VarMetaDataElements" />
    </xsl:call-template>
   </xsl:if>
  </xsl:for-each>
 </xsl:template>


 <!-- Filename and Pathname markers -->
 <!--                               -->

 <xsl:template name="Path-Without-Extension">
  <xsl:param name="ParamPath" />

  <!-- Determine extension -->
  <!--                     -->
  <xsl:variable name="VarExtension" select="wwfilesystem:GetExtension($ParamPath)" />

  <!-- Strip extension -->
  <!--                 -->
  <xsl:choose>
   <xsl:when test="string-length($VarExtension) &gt;= string-length($ParamPath)">
    <xsl:value-of select="''" />
   </xsl:when>

   <xsl:when test="string-length($VarExtension) &gt; 0">
    <xsl:value-of select="substring($ParamPath, 1, string-length($ParamPath) - string-length($VarExtension))" />
   </xsl:when>

   <xsl:otherwise>
    <xsl:value-of select="$ParamPath" />
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>

 <xsl:template name="Filename-Or-PathName-Marker">
  <xsl:param name="ParamNode" />
  <xsl:param name="ParamID" />
  <xsl:param name="ParamMarkerName" />

  <xsl:if test="(string-length($ParamNode/@xtrf) &gt; 0) and (string-length($ParamNode/parent::*/@xtrf) &gt; 0) and (contains($ParamNode/parent::*/@class, ' topic/topic '))">
   <!-- Determine base filename -->
   <!--                         -->
   <xsl:variable name="VarFilename">
    <xsl:choose>
     <xsl:when test="string-length($ParamNode/parent::*/@copy-to) &gt; 0">
      <xsl:value-of select="wwfilesystem:GetFileName($ParamNode/parent::*/@copy-to)" />
     </xsl:when>

     <xsl:otherwise>
      <xsl:value-of select="wwfilesystem:GetFileName($ParamNode/parent::*/@xtrf)" />
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>

   <!-- Base filename determined? -->
   <!--                           -->
   <xsl:if test="string-length($VarFilename) &gt; 0">
    <!-- Determine relative path without extension -->
    <!--                                           -->
    <xsl:variable name="VarFilenameWithoutExtension">
     <xsl:call-template name="Path-Without-Extension">
      <xsl:with-param name="ParamPath" select="$VarFilename" />
     </xsl:call-template>
    </xsl:variable>

    <!-- Emit marker -->
    <!--             -->
    <xsl:if test="string-length($VarFilenameWithoutExtension) &gt; 0">
     <wwdoc:Marker id="fm{$ParamID}" name="{$ParamMarkerName}">
      <wwdoc:TextRun id="fmtr{$ParamID}">
       <wwdoc:Text value="{$VarFilenameWithoutExtension}" />
      </wwdoc:TextRun>
     </wwdoc:Marker>
    </xsl:if>
   </xsl:if>
  </xsl:if>
 </xsl:template>

 <xsl:template name="Filename-Markers-Exist">
  <xsl:param name="ParamMetaDataElements" />

  <xsl:if test="count($ParamMetaDataElements/*[(contains(@class, ' topic/othermeta ')) and (@name = 'Filename') and (@wwtype:type = 'Marker') and (@outputclass = 'Filename')]) &gt; 0">
   <xsl:value-of select="true()" />
  </xsl:if>
 </xsl:template>

 <xsl:template name="Filename-Marker">
  <xsl:param name="ParamNode" />
  <xsl:param name="ParamID" />

  <xsl:call-template name="Filename-Or-PathName-Marker">
   <xsl:with-param name="ParamNode" select="$ParamNode" />
   <xsl:with-param name="ParamID" select="$ParamID" />
   <xsl:with-param name="ParamMarkerName" select="'Filename'" />
  </xsl:call-template>
 </xsl:template>

 <xsl:template name="Pathname-Marker">
  <xsl:param name="ParamNode" />
  <xsl:param name="ParamID" />

  <xsl:if test="(string-length($ParamNode/@xtrf) &gt; 0) and (string-length($ParamNode/parent::*/@xtrf) &gt; 0) and (contains($ParamNode/parent::*/@class, ' topic/topic '))">
   <!-- Determine relative path for pathname marker -->
   <!--                                             -->
   <xsl:variable name="VarRelativePath">
    <xsl:choose>
     <xsl:when test="string-length($ParamNode/parent::*/@copy-to) &gt; 0">
      <xsl:value-of select="$ParamNode/parent::*/@copy-to" />
     </xsl:when>

     <xsl:otherwise>
      <xsl:variable name="VarOriginalDocumentDirectoryPath">
       <xsl:value-of select="wwfilesystem:GetDirectoryName($ParameterOriginalDocumentPath)" />
       <xsl:text>\</xsl:text>
      </xsl:variable>

      <xsl:value-of select="substring-after($ParamNode/parent::*/@xtrf, $VarOriginalDocumentDirectoryPath)" />
     </xsl:otherwise>
    </xsl:choose>
   </xsl:variable>

   <!-- Path defined? -->
   <!--               -->
   <xsl:choose>
    <xsl:when test="string-length($VarRelativePath) &gt; 0">
     <!-- Determine relative path without extension -->
     <!--                                           -->
     <xsl:variable name="VarRelativePathWithoutExtension">
      <xsl:call-template name="Path-Without-Extension">
       <xsl:with-param name="ParamPath" select="$VarRelativePath" />
      </xsl:call-template>
     </xsl:variable>

     <!-- Emit marker -->
     <!--             -->
     <xsl:if test="string-length($VarRelativePathWithoutExtension) &gt; 0">
      <wwdoc:Marker id="pm{$ParamID}" name="Pathname">
       <wwdoc:TextRun id="pmtr{$ParamID}">
        <wwdoc:Text value="{$VarRelativePathWithoutExtension}" />
       </wwdoc:TextRun>
      </wwdoc:Marker>
     </xsl:if>
    </xsl:when>

    <xsl:otherwise>
     <!-- Process as filename marker -->
     <!--                            -->
     <xsl:call-template name="Filename-Or-PathName-Marker">
      <xsl:with-param name="ParamNode" select="$ParamNode" />
      <xsl:with-param name="ParamID" select="$ParamID" />
      <xsl:with-param name="ParamMarkerName" select="'Pathname'" />
     </xsl:call-template>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:if>
 </xsl:template>

 <!-- Using DITAVAL to flag or decorate text. Part 3 of 3.                         -->
 <!-- Call templates are further above, while the templates are defined here.      -->
 <!-- These templates have attributes defined with XSL-FO in mind. May need to     -->
 <!-- change them for HTML/Reverb/etc.                                             -->
 <!-- -Lief 14.June.2013                                                           -->

 <xsl:template name="Paragraph-Flagging-Color">
  <xsl:param name="ParamParagraph" />

  <xsl:variable name="VarStatusColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@color" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@color" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@color" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>


 <xsl:template name="Paragraph-Flagging-BackColor">
  <xsl:param name="ParamParagraph" />

  <xsl:variable name="VarStatusBackColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevBackColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsBackColor" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusBackColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusBackColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@backcolor" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="background-color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevBackColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevBackColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@backcolor" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="background-color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsBackColor) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsBackColor][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarColor" select="@backcolor" />
     <xsl:if test="string-length($VarColor) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="background-color" value="{$VarColor}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>


 <xsl:template name="Paragraph-Flagging-Style-Italics">
  <xsl:param name="ParamParagraph" />
  
  <xsl:variable name="VarStatusStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-style" value="italic" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-style" value="italic" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-style" value="italic" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>
 
 <xsl:template name="Paragraph-Flagging-Style-Bold">
  <xsl:param name="ParamParagraph" />
  
  <xsl:variable name="VarStatusStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsStyle" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-weight" value="{$VarStyle}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-weight" value="{$VarStyle}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsStyle) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsStyle][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarStyle" select="@style" />
     <xsl:if test="string-length($VarStyle) &gt; 0">
      <wwdoc:Style>
       <wwdoc:Attribute name="font-weight" value="{$VarStyle}" />
      </wwdoc:Style>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>
 
 
 <xsl:template name="Paragraph-Flagging-AltText">
  <xsl:param name="ParamParagraph" />
  
  <xsl:variable name="VarStatusAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/startflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value="{$VarAltText}: " />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/startflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value="{$VarAltText}: " />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/startflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value="{$VarAltText}: " />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>
 

  <xsl:template name="Paragraph-Flagging-AltEndText">
  <xsl:param name="ParamParagraph" />
  
  <xsl:variable name="VarStatusAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@status) &gt; 0][1]/@status " />
  <xsl:variable name="VarRevAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@rev) &gt; 0][1]/@rev " />
  <xsl:variable name="VarOtherPropsAltText" select="$ParamParagraph/ancestor-or-self::*[string-length(@otherprops) &gt; 0][1]/@otherprops " />
  <xsl:if test="string-length($VarStatusAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'status' and @val = $VarStatusAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/endflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value=" {$VarAltText}" />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarRevAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'rev' and @val = $VarRevAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/endflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value=" {$VarAltText}" />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
  <xsl:if test="string-length($VarOtherPropsAltText) &gt; 0">
   <xsl:variable name="VarPropNode" select="$GlobalDitaVal//*[@att = 'otherprops' and @val = $VarOtherPropsAltText][1]" />
   <xsl:for-each select="$VarPropNode">
    <xsl:variable name="VarAction" select="@action" />
    <xsl:if test="$VarAction = 'flag'">
     <xsl:variable name="VarAltText" select="$VarPropNode/endflag/alt-text/text()" />
     <xsl:if test="string-length($VarAltText) &gt; 0">
      <wwdoc:TextRun>
       <wwdoc:Text value=" {$VarAltText}" />
      </wwdoc:TextRun>
     </xsl:if>
    </xsl:if>
   </xsl:for-each>
  </xsl:if>
 </xsl:template>
 
 <!-- End of customization. -->
 
</xsl:stylesheet>

