<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
 <xsl:param name="GlobalDITAMapURI" />
 <xsl:param name="GlobalDITAMapExtension" />


 <xsl:output method="text" />


 <xsl:template match="/">
  <xsl:param name="ParamDITAMapURI" select="$GlobalDITAMapURI" />

  <!-- Process first topicref -->
  <!--                        -->
  <xsl:call-template name="ProcessTopicRef">
   <xsl:with-param name="ParamDITAMapURI" select="$ParamDITAMapURI" />
   <xsl:with-param name="ParamTopicRefNode" select="//*[contains(@class, ' map/topicref ') and (string-length(@href) &gt; 0)][1]" />
  </xsl:call-template>
 </xsl:template>

 <xsl:template match="*">
  <!-- Nothing to do -->
  <!--               -->
 </xsl:template>

 <xsl:template match="comment() | processing-instruction() | text()">
  <!-- Nothing to do -->
  <!--               -->
 </xsl:template>


 <xsl:template name="ProcessTopicRef">
  <xsl:param name="ParamDITAMapURI" />
  <xsl:param name="ParamTopicRefNode" />

  <!-- Determine DITA extension name -->
  <!--                               -->
  <xsl:variable name="VarDitaParsedExtensionName">
   <xsl:variable name="VarFileName">
    <xsl:call-template name="FileName">
     <xsl:with-param name="ParamPath" select="$ParamTopicRefNode/@href" />
    </xsl:call-template>
   </xsl:variable>

   <xsl:call-template name="Extension">
    <xsl:with-param name="ParamFileName" select="$VarFileName" />
   </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="VarDitaParsedExtension">
   <xsl:if test="string-length($VarDitaParsedExtensionName) &gt; 0">
    <xsl:text>.</xsl:text>
    <xsl:value-of select="$VarDitaParsedExtensionName" />
   </xsl:if>
  </xsl:variable>

  <!-- Inspect DITA map for extension -->
  <!--                                -->
  <xsl:variable name="VarDitaExtensionFromMap">
   <xsl:choose>
    <!-- DITA map reference? -->
    <!--                     -->
    <xsl:when test="$VarDitaParsedExtension = $GlobalDITAMapExtension">
     <!-- Resolve DITA map path -->
     <!--                       -->
     <xsl:variable name="VarDITAMapDirectoryPath">
      <xsl:call-template name="Parse-All-But-Last-Entry">
       <xsl:with-param name="ParamString" select="translate($ParamDITAMapURI, '\', '/')" />
       <xsl:with-param name="ParamDelimiter" select="'/'" />
      </xsl:call-template>
     </xsl:variable>
     <xsl:variable name="VarDITAMapURI">
      <xsl:value-of select="$VarDITAMapDirectoryPath" />
      <xsl:choose>
       <xsl:when test="contains($ParamTopicRefNode/@href, '#')">
        <xsl:value-of select="substring-before($ParamTopicRefNode/@href, '#')" />
       </xsl:when>

       <xsl:otherwise>
        <xsl:value-of select="$ParamTopicRefNode/@href" />
       </xsl:otherwise>
      </xsl:choose>
     </xsl:variable>

     <!-- Search map -->
     <!--            -->
     <xsl:apply-templates select="document($VarDITAMapURI)">
      <xsl:with-param name="ParamDITAMapURI" select="$VarDITAMapURI" />
     </xsl:apply-templates>
    </xsl:when>

    <!-- Use current answer -->
    <!--                    -->
    <xsl:otherwise>
     <xsl:value-of select="$VarDitaParsedExtension" />
    </xsl:otherwise>
   </xsl:choose>
  </xsl:variable>

  <!-- Emit DITA extension name or default -->
  <!--                                     -->
  <xsl:choose>
   <xsl:when test="string-length($VarDitaExtensionFromMap) &gt; 0">
    <xsl:value-of select="$VarDitaExtensionFromMap" />
   </xsl:when>

   <xsl:otherwise>
    <xsl:text>.xml</xsl:text>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>


 <xsl:template name="FileName">
  <xsl:param name="ParamPath" />

  <xsl:variable name="VarFileNameWithAnchor">
   <xsl:call-template name="Parse-Last-Entry">
    <xsl:with-param name="ParamString" select="translate($ParamPath, '\', '/')" />
    <xsl:with-param name="ParamDelimiter" select="'/'" />
   </xsl:call-template>
  </xsl:variable>

  <xsl:choose>
   <xsl:when test="contains($VarFileNameWithAnchor, '#')">
    <xsl:value-of select="substring-before($VarFileNameWithAnchor, '#')" />
   </xsl:when>

   <xsl:otherwise>
    <xsl:value-of select="$VarFileNameWithAnchor" />
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>


 <xsl:template name="Extension">
  <xsl:param name="ParamFileName" />

  <xsl:call-template name="Parse-Last-Entry">
   <xsl:with-param name="ParamString" select="$ParamFileName" />
   <xsl:with-param name="ParamDelimiter" select="'.'" />
  </xsl:call-template>
 </xsl:template>


 <xsl:template name="Parse-All-But-Last-Entry">
  <xsl:param name="ParamString" />
  <xsl:param name="ParamDelimiter" />

  <xsl:choose>
   <xsl:when test="contains($ParamString, $ParamDelimiter)">
    <!-- Emit prefix -->
    <!--             -->
    <xsl:value-of select="substring-before($ParamString, $ParamDelimiter)" />
    <xsl:value-of select="$ParamDelimiter" />

    <xsl:call-template name="Parse-All-But-Last-Entry">
     <xsl:with-param name="ParamString" select="substring-after($ParamString, $ParamDelimiter)" />
     <xsl:with-param name="ParamDelimiter" select="$ParamDelimiter" />
    </xsl:call-template>
   </xsl:when>

   <xsl:otherwise>
    <!-- Done! -->
    <!--       -->
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>


 <xsl:template name="Parse-Last-Entry">
  <xsl:param name="ParamString" />
  <xsl:param name="ParamDelimiter" />

  <xsl:choose>
   <xsl:when test="contains($ParamString, $ParamDelimiter)">
    <xsl:call-template name="Parse-Last-Entry">
     <xsl:with-param name="ParamString" select="substring-after($ParamString, $ParamDelimiter)" />
     <xsl:with-param name="ParamDelimiter" select="$ParamDelimiter" />
    </xsl:call-template>
   </xsl:when>

   <xsl:otherwise>
    <xsl:value-of select="$ParamString" />
   </xsl:otherwise>
  </xsl:choose>
 </xsl:template>
</xsl:stylesheet>
